# Software  Project
Graph Embedding Propagation on WordNet

Softwareprojekt WS2018/19 

Institute of Computational Linguistics

Heidelberg University, Germany

# Authors
Nadia Arslan, Lyuba Dimitrova, Utaemon Toyota, Nicolas Weber

# Supervisor
Prof. Dr. Anette Frank

# Overview
This project provides our re-implementation of the Graph Embedding Propagation (EP) algorithm based on the approach of Duran and Niepert, 2017. Our algorithm EP-SP was applied to the the task (1) of node classification over the cora dataset and to the task (2) of Word Sense Disambiguation (WSD) over WordNet 3.0.
(1) provides an evaluation of our graph and makes a comparison to the results presented in the original paper possible.
(2) provides two approaches for WSD using synset embeddings from applying EP-SP to WordNet 3.0. The task operates over the English All-Word datasets of SensEval-2 and SensEval-3.
For more details please read our project report.

# Project structure

    |_ cli.py
    |	 The command line interface for our project. Imports and runs the files in scripts/. See Usage for more info.
    |
    |_ data/
    |    All necessary inputs and the generated outputs.
    |	 |
    |	 |_ cora
    |    |    |_embeddings - the output of EP on Cora
    |    |    |_graph - the input for EP - a pickled networkX graph
    |    |    |_models - the saved TF models; can be used to save the embeddings after 40 epochs, for instance
    |    |    |_raw - the raw Cora data, input for the Cora preprocessing and the node classification
    |    |    |_summaries - the TF loss summaries for the two Cora label types; used to produce the figures in the report
    |    |
    |    |_ other - files that don't belong to a particular dataset
    |    |
    |    |_ senseval2/senseval3
    |    |    |_processed - the processed raw S2 or S3 data; input for the WSD
    |    |    |_raw - the raw S2 or S3 data
    |    |    |_wsd-answers - outputs of the WSD on the S2 or S3 data
    |    |
    |	 |_ wordnet
    |         |_embeddings - the output of EP on WordNet
    |         |_graph - the input for EP - a pickled networkX graph
    |         |_models - the saved TF models; can be used to save the embeddings after 50 epochs, for instance
    |         |_raw - the raw WN data, input for the preprocessing scripts
    |         |_summaries - the TF loss summaries for the five WN label types; used to produce the figures in the report
    |         |_mappings - various mappings for the synset IDs, lemmata, WN3.0->WN1.7 etc.; used in the WSD
    |
    |_ __init__.py
    |
    |_ README.md
    |    This file.
    |
    |_ requirements.txt
    |      An image of the virtualenv. -> pip install -r requirements.txt
    |
    |_ scripts/
    |    Python scripts for EP, preprocessing, node classification and WSD. 
    |    |
    |    |_ embedding_propagation - the EP algorithm
    |    |_ node_classification - the NC experiment on the Cora dataset
    |    |_ preprocessing - preprocessing scripts for Cora, SensEval, and WordNet. Not part of the CLI, so partly with aux. files.
    |    |_ scoring - the official S2 and S3 All Words Task scorer for the WSD
    |    |_ wsd - the two WSD methods
    |
    

# Usage
The CLI provides 6 different commands.

List the commands: 
    python[3] cli.py --help

    1. embedding-propagation
	Runs EP on an input graph and saves the learnt embeddings.
	Help and options: python[3] cli.py embedding-propagation --help
	Example usage: python[3] cli.py embedding-propagation      // runs on the WordNet graph by default
		       python[3] cli.py embedding-propagation -i data/cora/graphs/cora_graph.pkl -o data/cora/ -b 64  // to run on Cora

    2. node-classification
	Runs the NC experiment and print the results on the console.
	Help and options: python[3] cli.py node-classification --help
	Example usage: python[3] cli.py node-classification -s 10      // only one experiment, using 10 for the random seed
	               python[3] cli.py node-classification -i 50      // 50 runs with different random seeds

    3. process-cora
	Creates a networkx graph from the raw Cora data.
	Help and options: python[3] cli.py process-cora --help
	Example usage: python[3] cli.py process-cora -o test/graph.pkl

    4. score-wsd
	Calls the official WSD English All Word Task scorer.
	Help and options: python[3] cli.py score-wsd --help
	Example usage: python[3] cli.py score-wsd -s 3 -a wsd2_5

    5. wsd-1
	Calls the WSD method #1.
	Help and options: python[3] cli.py wsd-1 --help
	Example usage: python[3] cli.py wsd-1 -o wsd1    // after, you can call python[3] cli.py score-wsd -a wsd1 in order to score it

    6. wsd-2
	Calls the WSD method #2.
	Help and options: python[3] cli.py wsd-2 --help
	Example usage: python[3] cli.py wsd-2 -c 15 -o wsd2    // after, you can call python[3] cli.py score-wsd -a wsd2_15 in order to score it


# Licenses
This software is distributed under the MIT License.

MIT License

Copyright (c) 2019 Nadia Arslan, Lyuba Dimitrova, Utaemon Toyota, Nicolas Weber

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.


## Tensorflow
   Copyright 2017, The TensorFlow Authors.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
limitations under the License.


## WordNet
WordNet Release 3.0

This software and database is being provided to you, the LICENSEE, by  
Princeton University under the following license.  By obtaining, using  
and/or copying this software and database, you agree that you have  
read, understood, and will comply with these terms and conditions.:  
  
Permission to use, copy, modify and distribute this software and  
database and its documentation for any purpose and without fee or  
royalty is hereby granted, provided that you agree to comply with  
the following copyright notice and statements, including the disclaimer,  
and that the same appear on ALL copies of the software, database and  
documentation, including modifications that you make for internal  
use or for distribution.  
  
WordNet 3.0 Copyright 2006 by Princeton University.  All rights reserved.  
  
THIS SOFTWARE AND DATABASE IS PROVIDED "AS IS" AND PRINCETON  
UNIVERSITY MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR  
IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, PRINCETON  
UNIVERSITY MAKES NO REPRESENTATIONS OR WARRANTIES OF MERCHANT-  
ABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE  
OF THE LICENSED SOFTWARE, DATABASE OR DOCUMENTATION WILL NOT  
INFRINGE ANY THIRD PARTY PATENTS, COPYRIGHTS, TRADEMARKS OR  
OTHER RIGHTS.  
  
The name of Princeton University or Princeton may not be used in  
advertising or publicity pertaining to distribution of the software  
and/or database.  Title to copyright in this software, database and  
any associated documentation shall at all times remain with  
Princeton University and LICENSEE agrees to preserve same.  


## NetworkX
NetworkX is distributed with the 3-clause BSD license.

::

   Copyright (C) 2004-2018, NetworkX Developers
   Aric Hagberg <hagberg@lanl.gov>
   Dan Schult <dschult@colgate.edu>
   Pieter Swart <swart@lanl.gov>
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the name of the NetworkX Developers nor the names of its
       contributors may be used to endorse or promote products derived
       from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


## SpaCy
The MIT License (MIT)

Copyright (C) 2016 ExplosionAI UG (haftungsbeschränkt), 2016 spaCy GmbH, 2015 Matthew Honnibal

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.


## NLTK
Copyright (C) 2001-2019 NLTK Project

Licensed under the Apache License, Version 2.0 (the 'License');
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an 'AS IS' BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


## Scikit-Learn
New BSD License

Copyright (c) 2007–2019 The scikit-learn developers.
All rights reserved.


Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  a. Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
  b. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  c. Neither the name of the Scikit-learn Developers  nor the names of
     its contributors may be used to endorse or promote products
     derived from this software without specific prior written
     permission. 


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
